@@ -18,6 +18,8 @@
  *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
+#include "config.h"
+
 #include <errno.h>
 #include <iconv.h>
 #include <unistd.h>
@@ -27,37 +29,41 @@
 
 #include <fcitx/fcitx.h>
 #include <fcitx/module.h>
-#include <fcitx/instance.h>
 #include <fcitx/hook.h>
 #include <fcitx-utils/log.h>
 #include <fcitx/candidate.h>
 #include <fcitx-config/xdg.h>
-#include <fcitx/module/pinyin/pydef.h>
+#include <fcitx/module/pinyin/fcitx-pinyin.h>
 
-#include "config.h"
 #include "cloudpinyin.h"
 #include "fetch.h"
+#include "parse.h"
 
-#define CHECK_VALID_IM (im && \
+DEFINE_GET_ADDON("fcitx-sunpinyin", SunPinyin)
+DEFINE_GET_ADDON("fcitx-libpinyin", LibPinyin)
+DEFINE_GET_ADDON("fcitx-sogoupinyin", SogouPinyin)
+DEFINE_GET_AND_INVOKE_FUNC(SunPinyin, GetFullPinyin, 0)
+DEFINE_GET_AND_INVOKE_FUNC(SunPinyin, AddWord, 1)
+
+// Maybe not the right name, but doesn't matter....
+DEFINE_GET_AND_INVOKE_FUNC(LibPinyin, AddWord, 0)
+DEFINE_GET_AND_INVOKE_FUNC(SogouPinyin, AddWord, 0)
+
+#define CHECK_VALID_IM (im &&                                 \
+                        strcmp(im->langCode, "zh_CN") == 0 && \
                         (strcmp(im->uniqueName, "pinyin") == 0 || \
                         strcmp(im->uniqueName, "pinyin-libpinyin") == 0 || \
                         strcmp(im->uniqueName, "shuangpin-libpinyin") == 0 || \
                         strcmp(im->uniqueName, "googlepinyin") == 0 || \
                         strcmp(im->uniqueName, "sunpinyin") == 0 || \
-                        strcmp(im->uniqueName, "shuangpin") == 0))
+                        strcmp(im->uniqueName, "shuangpin") == 0 || \
+                        strcmp(im->uniqueName, "sogou-pinyin") == 0))
 
 #define CLOUDPINYIN_CHECK_PAGE_NUMBER 3
 
-#define LOGLEVEL DEBUG
-
-#ifdef LIBICONV_SECOND_ARGUMENT_IS_CONST
-typedef const char* IconvStr;
-#else
-typedef char* IconvStr;
-#endif
-
 typedef struct _CloudCandWord {
     boolean filled;
+    uint64_t timestamp;
 } CloudCandWord;
 
 typedef struct _CloudPinyinEngine {
@@ -84,18 +90,12 @@ static void _CloudPinyinAddCandidateWord(FcitxCloudPinyin* cloudpinyin, const ch
 static void CloudPinyinFillCandidateWord(FcitxCloudPinyin* cloudpinyin, const char* pinyin);
 static boolean LoadCloudPinyinConfig(FcitxCloudPinyinConfig* fs);
 static void SaveCloudPinyinConfig(FcitxCloudPinyinConfig* fs);
-static char *GetCurrentString(FcitxCloudPinyin* cloudpinyin);
-static char* SplitHZAndPY(char* string);
+static char *GetCurrentString(FcitxCloudPinyin* cloudpinyin,
+                              char **ascii_part);
 static void CloudPinyinHookForNewRequest(void* arg);
 static CURL* CloudPinyinGetFreeCurlHandle(FcitxCloudPinyin* cloudpinyin);
-static void CloudPinyinReleaseCurlHandle(FcitxCloudPinyin* cloudpinyin, CURL* curl);
-
-void SogouParseKey(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue);
-char* SogouParsePinyin(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue);
-void QQParseKey(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue);
-char* QQParsePinyin(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue);
-char* GoogleParsePinyin(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue);
-char* BaiduParsePinyin(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue);
+static void CloudPinyinReleaseCurlHandle(FcitxCloudPinyin* cloudpinyin,
+                                         CURL* curl);
 
 CloudPinyinEngine engine[4] =
 {
@@ -128,39 +128,45 @@ CloudPinyinEngine engine[4] =
 
 CONFIG_DESC_DEFINE(GetCloudPinyinConfigDesc, "fcitx-cloudpinyin.desc")
 
-FCITX_EXPORT_API
-FcitxModule module = {
-    CloudPinyinCreate,
-    CloudPinyinSetFD,
-    CloudPinyinProcessEvent,
-    CloudPinyinDestroy,
-    CloudPinyinReloadConfig
+FCITX_DEFINE_PLUGIN(fcitx_cloudpinyin, module, FcitxModule) = {
+    .Create = CloudPinyinCreate,
+    .Destroy = CloudPinyinDestroy,
+    .SetFD = CloudPinyinSetFD,
+    .ProcessEvent = CloudPinyinProcessEvent,
+    .ReloadConfig = CloudPinyinReloadConfig
 };
 
-FCITX_EXPORT_API
-int ABI_VERSION = FCITX_ABI_VERSION;
-
-static inline boolean ishex(char ch)
+static uint64_t
+CloudGetTimeStamp()
 {
-    if ((ch >= '0' && ch <= '9') || (ch >='a' && ch <='f') || (ch >='A' && ch <='F'))
-        return true;
-    return false;
+    struct timeval current_time;
+    gettimeofday(&current_time, NULL);
+    return (((uint64_t)current_time.tv_sec * 1000)
+            + (current_time.tv_usec / 1000));
 }
 
-static inline unsigned char tohex(char ch)
+static void
+CloudSetClientPreedit(FcitxCloudPinyin *cloudpinyin, const char *str)
 {
-    if (ch >= '0' && ch <= '9')
-        return ch - '0';
-    if (ch >='a' && ch <='f')
-        return ch - 'a' + 10;
-    if (ch >='A' && ch <='F')
-        return ch - 'A' + 10;
-    return 0;
+    FcitxInputState *input = FcitxInstanceGetInputState(cloudpinyin->owner);
+    FcitxMessages *message = FcitxInputStateGetClientPreedit(input);
+    char *py;
+    char *string = GetCurrentString(cloudpinyin, &py);
+    FcitxMessagesSetMessageCount(message, 0);
+    if (py) {
+        *py = '\0';
+        FcitxMessagesAddMessageAtLast(message, MSG_INPUT, "%s%s", string, str);
+    } else {
+        FcitxMessagesAddMessageAtLast(message, MSG_INPUT, "%s", str);
+    }
+    fcitx_utils_free(string);
+    FcitxInstanceUpdateClientSideUI(
+        cloudpinyin->owner, FcitxInstanceGetCurrentIC(cloudpinyin->owner));
 }
 
 void* CloudPinyinCreate(FcitxInstance* instance)
 {
-    FcitxCloudPinyin* cloudpinyin = fcitx_utils_malloc0(sizeof(FcitxCloudPinyin));
+    FcitxCloudPinyin *cloudpinyin = fcitx_utils_new(FcitxCloudPinyin);
     bindtextdomain("fcitx-cloudpinyin", LOCALEDIR);
     cloudpinyin->owner = instance;
     int pipe1[2];
@@ -276,13 +282,11 @@ void CloudPinyinAddCandidateWord(void* arg)
         if (FcitxInputStateGetRawInputBufferSize(input) >= cloudpinyin->config.iMinimumPinyinLength)
         {
             char* strToFree = NULL, *inputString;
-            strToFree = GetCurrentString(cloudpinyin);
-            inputString = SplitHZAndPY(strToFree);
+            strToFree = GetCurrentString(cloudpinyin, &inputString);
 
-            if (inputString)
-            {
+            if (inputString) {
                 CloudPinyinCache* cacheEntry = CloudPinyinCacheLookup(cloudpinyin, inputString);
-                FcitxLog(LOGLEVEL, "%s", inputString);
+                FcitxLog(DEBUG, "%s", inputString);
                 if (cacheEntry == NULL)
                     CloudPinyinAddInputRequest(cloudpinyin, inputString);
                 _CloudPinyinAddCandidateWord(cloudpinyin, inputString);
@@ -350,13 +354,19 @@ void CloudPinyinSetFD(void* arg)
 void CloudPinyinProcessEvent(void* arg)
 {
     FcitxCloudPinyin* cloudpinyin = (FcitxCloudPinyin*) arg;
+    FcitxInstance* instance = cloudpinyin->owner;
+    if (!FD_ISSET(cloudpinyin->pipeRecv, FcitxInstanceGetReadFDSet(instance)))
+        return;
+
     char c;
     while (read(cloudpinyin->pipeRecv, &c, sizeof(char)) > 0);
     pthread_mutex_lock(&cloudpinyin->finishQueueLock);
     CurlQueue* queue;
     queue = cloudpinyin->finishQueue;
+    /* this queue header is empty, so the check condition is "next" not null */
     while (queue->next != NULL)
     {
+        /* remove pivot from queue, thus pivot need to be free'd in HandleRequest */
         CurlQueue* pivot = queue->next;
         queue->next = queue->next->next;
         CloudPinyinHandleRequest(cloudpinyin, pivot);
@@ -366,7 +376,48 @@ void CloudPinyinProcessEvent(void* arg)
 
 void CloudPinyinDestroy(void* arg)
 {
+    FcitxCloudPinyin* cloudpinyin = (FcitxCloudPinyin*) arg;
+    char c = 1;
+    write(cloudpinyin->pipeNotify, &c, sizeof(char));
+    pthread_join(cloudpinyin->pid, NULL);
+    pthread_mutex_destroy(&cloudpinyin->pendingQueueLock);
+    pthread_mutex_destroy(&cloudpinyin->finishQueueLock);
+    while (cloudpinyin->cache)
+    {
+        CloudPinyinCache* head = cloudpinyin->cache;
+        HASH_DEL(cloudpinyin->cache, cloudpinyin->cache);
+        free(head->pinyin);
+        free(head->str);
+        free(head);
+    }
 
+    close(cloudpinyin->pipeRecv);
+    close(cloudpinyin->pipeNotify);
+
+    close(cloudpinyin->fetch->pipeRecv);
+    close(cloudpinyin->fetch->pipeNotify);
+    int i = 0;
+    for (i = 0; i < MAX_HANDLE; i ++) {
+        if (cloudpinyin->freeList[i].curl) {
+            curl_easy_cleanup(cloudpinyin->freeList[i].curl);
+        }
+    }
+
+    curl_multi_cleanup(cloudpinyin->fetch->curlm);
+#define _FREE_QUEUE(NAME) \
+    while(NAME) { \
+        CurlQueue* queue = NAME; \
+        NAME = NAME->next; \
+        fcitx_utils_free(queue->str); \
+        fcitx_utils_free(queue->pinyin); \
+        free(queue); \
+    }
+    _FREE_QUEUE(cloudpinyin->pendingQueue)
+    _FREE_QUEUE(cloudpinyin->finishQueue)
+    _FREE_QUEUE(cloudpinyin->fetch->queue)
+    FcitxConfigFree(&cloudpinyin->config.config);
+    free(cloudpinyin->fetch);
+    free(cloudpinyin);
 }
 
 void CloudPinyinReloadConfig(void* arg)
@@ -445,12 +496,10 @@ void CloudPinyinHandleRequest(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue)
                 FcitxIM* im = FcitxInstanceGetCurrentIM(cloudpinyin->owner);
 
                 char* strToFree = NULL, *inputString;
-                strToFree = GetCurrentString(cloudpinyin);
-                inputString = SplitHZAndPY(strToFree);
+                strToFree = GetCurrentString(cloudpinyin, &inputString);
 
-                if (inputString)
-                {
-                    FcitxLog(LOGLEVEL, "fill: %s %s", inputString, queue->pinyin);
+                if (inputString) {
+                    FcitxLog(DEBUG, "fill: %s %s", inputString, queue->pinyin);
                     if (strcmp(inputString, queue->pinyin) == 0)
                     {
                         if (CHECK_VALID_IM)
@@ -477,10 +526,8 @@ void CloudPinyinHandleRequest(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue)
         }
     }
     CloudPinyinReleaseCurlHandle(cloudpinyin, queue->curl);
-    if (queue->str)
-        free(queue->str);
-    if (queue->pinyin)
-        free(queue->pinyin);
+    fcitx_utils_free(queue->str);
+    fcitx_utils_free(queue->pinyin);
     free(queue);
 }
 
@@ -546,35 +593,46 @@ void _CloudPinyinAddCandidateWord(FcitxCloudPinyin* cloudpinyin, const char* pin
 {
     CloudPinyinCache* cacheEntry = CloudPinyinCacheLookup(cloudpinyin, pinyin);
     FcitxInputState* input = FcitxInstanceGetInputState(cloudpinyin->owner);
-    struct _FcitxCandidateWordList* candList = FcitxInputStateGetCandidateList(input);
+    FcitxCandidateWordList* candList = FcitxInputStateGetCandidateList(input);
+
+    int order = (cloudpinyin->config.iCandidateOrder <= 2) ?
+        1 : (cloudpinyin->config.iCandidateOrder - 1);
 
     if (cacheEntry) {
         FcitxCandidateWord* cand;
         /* only check the first three page */
-        int size = FcitxCandidateWordGetPageSize(candList) * CLOUDPINYIN_CHECK_PAGE_NUMBER;
-        int i = 0;
-        for (cand = FcitxCandidateWordGetFirst(FcitxInputStateGetCandidateList(input));
-             cand != NULL;
-             cand = FcitxCandidateWordGetNext(FcitxInputStateGetCandidateList(input), cand))
-        {
-            if (strcmp(cand->strWord, cacheEntry->str) == 0)
+        int pagesize = FcitxCandidateWordGetPageSize(candList);
+        int size = pagesize * CLOUDPINYIN_CHECK_PAGE_NUMBER;
+        int i;
+        if (cloudpinyin->config.iCandidateOrder <= 1) {
+            order = 0;
+        }
+        for (i = 0;i < size &&
+                 (cand = FcitxCandidateWordGetByTotalIndex(candList, i));i++) {
+            if (strcmp(cand->strWord, cacheEntry->str) == 0) {
+                if (i > order && i >= pagesize) {
+                    FcitxCandidateWordMoveByWord(candList, cand, order);
+                    if (order == 0) {
+                        CloudSetClientPreedit(cloudpinyin, cacheEntry->str);
+                    }
+                }
                 return;
-            i ++;
-            if (i >= size)
-                break;
+            }
+        }
+        if (order == 0) {
+            CloudSetClientPreedit(cloudpinyin, cacheEntry->str);
         }
     }
 
     FcitxCandidateWord candWord;
     CloudCandWord* cloudCand = fcitx_utils_malloc0(sizeof(CloudCandWord));
-    if (cacheEntry)
-    {
+    if (cacheEntry) {
         cloudCand->filled = true;
+        cloudCand->timestamp = 0;
         candWord.strWord = strdup(cacheEntry->str);
-    }
-    else
-    {
+    } else {
         cloudCand->filled = false;
+        cloudCand->timestamp = CloudGetTimeStamp();
         candWord.strWord = strdup("..");
     }
 
@@ -589,25 +647,24 @@ void _CloudPinyinAddCandidateWord(FcitxCloudPinyin* cloudpinyin, const char* pin
         candWord.extraType = MSG_TIPS;
     }
 
-    int order = cloudpinyin->config.iCandidateOrder - 1;
-    if (order < 0)
-        order = 0;
-
     FcitxCandidateWordInsert(candList, &candWord, order);
 }
 
-void CloudPinyinFillCandidateWord(FcitxCloudPinyin* cloudpinyin, const char* pinyin)
+#define LOADING_TIME_QUICK_THRESHOLD 300
+#define DUP_PLACE_HOLDER "\xe2\x98\xba"
+
+void CloudPinyinFillCandidateWord(FcitxCloudPinyin* cloudpinyin,
+                                  const char* pinyin)
 {
     CloudPinyinCache* cacheEntry = CloudPinyinCacheLookup(cloudpinyin, pinyin);
     FcitxInputState* input = FcitxInstanceGetInputState(cloudpinyin->owner);
-    struct _FcitxCandidateWordList* candList = FcitxInputStateGetCandidateList(input);
-    if (cacheEntry)
-    {
-        FcitxCandidateWord* candWord;
-        for (candWord = FcitxCandidateWordGetFirst(candList);
-             candWord != NULL;
-             candWord = FcitxCandidateWordGetNext(candList, candWord))
-        {
+    FcitxCandidateWordList* candList = FcitxInputStateGetCandidateList(input);
+    if (cacheEntry) {
+        int cloudidx;
+        FcitxCandidateWord *candWord;
+        for (cloudidx = 0;
+             (candWord = FcitxCandidateWordGetByTotalIndex(candList, cloudidx));
+             cloudidx++) {
             if (candWord->owner == cloudpinyin)
                 break;
         }
@@ -619,31 +676,56 @@ void CloudPinyinFillCandidateWord(FcitxCloudPinyin* cloudpinyin, const char* pin
         if (cloudCand->filled)
             return;
 
-        FcitxCandidateWord* cand;
-        int i = 0;
-        int size = FcitxCandidateWordGetPageSize(candList) * CLOUDPINYIN_CHECK_PAGE_NUMBER;
-        for (cand = FcitxCandidateWordGetFirst(candList);
-             cand != NULL;
-             cand = FcitxCandidateWordGetNext(candList, cand))
-        {
+        FcitxCandidateWord *cand;
+        int i;
+        int pagesize = FcitxCandidateWordGetPageSize(candList);
+        int size = pagesize * CLOUDPINYIN_CHECK_PAGE_NUMBER;
+        for (i = 0;i < size &&
+                 (cand = FcitxCandidateWordGetByTotalIndex(candList, i));i++) {
             if (strcmp(cand->strWord, cacheEntry->str) == 0) {
                 FcitxCandidateWordRemove(candList, candWord);
+                /* if cloud word is not on the first page.. impossible */
+                if (cloudidx < pagesize) {
+                    /* if the duplication before cloud word */
+                    if (i < cloudidx) {
+                        if (CloudGetTimeStamp() - cloudCand->timestamp
+                            > LOADING_TIME_QUICK_THRESHOLD) {
+                            FcitxCandidateWordInsertPlaceHolder(candList, cloudidx);
+                            FcitxCandidateWord* placeHolder = FcitxCandidateWordGetByTotalIndex(candList, cloudidx);
+                            if (placeHolder && placeHolder->strWord == NULL)
+                                placeHolder->strWord = strdup(DUP_PLACE_HOLDER);
+                        }
+                    } else {
+                        if (i >= pagesize) {
+                            FcitxCandidateWordMove(candList, i - 1, cloudidx);
+                        } else {
+                            if (CloudGetTimeStamp() - cloudCand->timestamp
+                                > LOADING_TIME_QUICK_THRESHOLD) {
+                                FcitxCandidateWordInsertPlaceHolder(candList, cloudidx);
+                                FcitxCandidateWord* placeHolder = FcitxCandidateWordGetByTotalIndex(candList, cloudidx);
+                                if (placeHolder && placeHolder->strWord == NULL)
+                                    placeHolder->strWord = strdup(DUP_PLACE_HOLDER);
+                            }
+                        }
+                    }
+                }
                 FcitxUIUpdateInputWindow(cloudpinyin->owner);
                 candWord = NULL;
                 break;
             }
-            i ++;
-            if (i >= size)
-                break;
         }
 
-        if (candWord)
-        {
-            if (cloudCand->filled == false)
-            {
+        if (candWord) {
+            if (cloudCand->filled == false) {
                 cloudCand->filled = true;
                 free(candWord->strWord);
                 candWord->strWord = strdup(cacheEntry->str);
+                if (cloudpinyin->config.iCandidateOrder <= 1 &&
+                    (CloudGetTimeStamp() - cloudCand->timestamp
+                     <= LOADING_TIME_QUICK_THRESHOLD)) {
+                    FcitxCandidateWordMoveByWord(candList, candWord, 0);
+                    CloudSetClientPreedit(cloudpinyin, cacheEntry->str);
+                }
                 FcitxUIUpdateInputWindow(cloudpinyin->owner);
             }
         }
@@ -657,31 +739,39 @@ INPUT_RETURN_VALUE CloudPinyinGetCandWord(void* arg, FcitxCandidateWord* candWor
     FcitxInputState* input = FcitxInstanceGetInputState(cloudpinyin->owner);
     if (cloudCand->filled)
     {
-        char* string = GetCurrentString(cloudpinyin);
-        char* py = SplitHZAndPY(string);
-        if (py)
-        {
+        char *py;
+        char *string = GetCurrentString(cloudpinyin, &py);
+        if (py) {
             *py = 0;
 
-            snprintf(FcitxInputStateGetOutputString(input), MAX_USER_INPUT, "%s%s", string, candWord->strWord);
+            snprintf(FcitxInputStateGetOutputString(input),
+                     MAX_USER_INPUT, "%s%s", string, candWord->strWord);
 
             FcitxIM* im = FcitxInstanceGetCurrentIM(cloudpinyin->owner);
-            FcitxModuleFunctionArg args;
-            args.args[0] = FcitxInputStateGetOutputString(input);
-            if (im)
-            {
-                if (strcmp(im->uniqueName, "sunpinyin") == 0)
-                    FcitxModuleInvokeFunctionByName(cloudpinyin->owner, "fcitx-sunpinyin", 1, args);
-                else if (strcmp(im->uniqueName, "shuangpin") == 0 || strcmp(im->uniqueName, "pinyin") == 0)
-                    FcitxModuleInvokeFunctionByName(cloudpinyin->owner, "fcitx-pinyin", 7, args);
+            if (im) {
+                char *output_string = FcitxInputStateGetOutputString(input);
+                FCITX_DEF_MODULE_ARGS(args, output_string);
+                if (strcmp(im->uniqueName, "sunpinyin") == 0) {
+                    FcitxSunPinyinInvokeAddWord(cloudpinyin->owner, args);
+                } else if (strcmp(im->uniqueName, "shuangpin") == 0 ||
+                           strcmp(im->uniqueName, "pinyin") == 0) {
+                    FcitxPinyinInvokeAddUserPhrase(cloudpinyin->owner, args);
+                } else if (strcmp(im->uniqueName, "pinyin-libpinyin") == 0 ||
+                           strcmp(im->uniqueName, "shuangpin-libpinyin") == 0) {
+                    FcitxLibPinyinInvokeAddWord(cloudpinyin->owner, args);
+                }
+                else if (strcmp(im->uniqueName, "sogou-pinyin") == 0)
+                {
+                    FcitxSogouPinyinInvokeAddWord(cloudpinyin->owner, args);
+                }
             }
         }
         if (string)
             free(string);
         return IRV_COMMIT_STRING;
-    }
-    else
+    } else {
         return IRV_DO_NOTHING;
+    }
 }
 
 
@@ -727,110 +817,74 @@ void SaveCloudPinyinConfig(FcitxCloudPinyinConfig* fs)
         fclose(fp);
 }
 
-char *GetCurrentString(FcitxCloudPinyin* cloudpinyin)
+char *GetCurrentString(FcitxCloudPinyin* cloudpinyin, char **ascii_part)
 {
     FcitxIM* im = FcitxInstanceGetCurrentIM(cloudpinyin->owner);
-    if (!im)
+    if (!im) {
+        *ascii_part = NULL;
         return NULL;
+    }
     FcitxInputState* input = FcitxInstanceGetInputState(cloudpinyin->owner);
     char* string = FcitxUIMessagesToCString(FcitxInputStateGetPreedit(input));
     char p[MAX_USER_INPUT + 1], *pinyin, *lastpos;
-    pinyin = SplitHZAndPY(string);
+    pinyin = fcitx_utils_get_ascii_part(string);
     lastpos = pinyin;
     boolean endflag;
     int hzlength = pinyin - string;
     size_t plength = hzlength;
     strncpy(p, string, hzlength);
     p[hzlength] = '\0';
-    do
-    {
+    do {
         endflag = (*pinyin != '\0');
-
-        if (*pinyin == ' ' || *pinyin == '\'' || *pinyin == '\0')
-        {
+        if (*pinyin == ' ' || *pinyin == '\'' || *pinyin == '\0') {
             *pinyin = 0;
 
-            if (*lastpos != '\0')
-            {
+            if (*lastpos != '\0') {
                 char* result = NULL;
-                FcitxModuleFunctionArg arg;
-                arg.args[0] = lastpos;
                 boolean isshuangpin = false;
-                if (strcmp(im->uniqueName, "sunpinyin") == 0)
-                {
-                    boolean issp = false;
-                    arg.args[1] = &issp;
-                    result = FcitxModuleInvokeFunctionByName(cloudpinyin->owner, "fcitx-sunpinyin", 0, arg);
-                    isshuangpin = issp;
-                }
-                else if (strcmp(im->uniqueName, "shuangpin") == 0)
-                {
+                if (strcmp(im->uniqueName, "sunpinyin") == 0) {
+                    FCITX_DEF_MODULE_ARGS(args, lastpos, &isshuangpin);
+                    result = FcitxSunPinyinInvokeGetFullPinyin(
+                        cloudpinyin->owner, args);
+                } else if (strcmp(im->uniqueName, "shuangpin") == 0) {
                     isshuangpin = true;
-                    result = InvokeFunction(cloudpinyin->owner, FCITX_PINYIN, SP2QP, arg);
+                    result = FcitxPinyinSP2QP(cloudpinyin->owner, lastpos);
                 }
-                if (isshuangpin)
-                {
-                    if (result)
-                    {
-                        if (plength + strlen(result) < MAX_USER_INPUT)
-                        {
+                if (isshuangpin) {
+                    if (result) {
+                        if (plength + strlen(result) < MAX_USER_INPUT) {
                             strcat(p + plength, result);
                             plength += strlen(result);
                             free(result);
-                        }
-                        else
-                        {
+                        } else {
                             p[hzlength] = '\0';
                             break;
                         }
                     }
-                }
-                else
-                {
-                    if (plength + strlen(lastpos) < MAX_USER_INPUT)
-                    {
+                } else {
+                    if (plength + strlen(lastpos) < MAX_USER_INPUT) {
                         strcat(p + plength, lastpos);
                         plength += strlen(lastpos);
-                    }
-                    else
-                    {
+                    } else {
                         p[hzlength] = '\0';
                         break;
                     }
                 }
             }
-
             lastpos = pinyin + 1;
         }
         pinyin ++;
-
     } while(endflag);
     free(string);
     /* no pinyin append, return NULL for off it */
-    if (p[hzlength] == '\0')
+    if (p[hzlength] == '\0') {
+        *ascii_part = NULL;
         return NULL;
-    else
-        return strdup(p);
-}
-
-char* SplitHZAndPY(char* string)
-{
-    if (string == NULL)
-        return NULL;
-
-    char* s = string;
-    while (*s)
-    {
-        char* p;
-        int chr;
-
-        p = fcitx_utf8_get_char(s, &chr);
-        if (p - s == 1)
-            break;
-        s = p;
+    } else {
+        char *res = strdup(p);
+        *ascii_part = res + hzlength;
+        return res;
     }
-
-    return s;
 }
 
 void SogouParseKey(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue)
@@ -839,10 +893,8 @@ void SogouParseKey(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue)
     const char* ime_patch_key = "ime_patch_key = \"";
     size_t len = strlen(str);
     if (len == SOGOU_KEY_LENGTH + strlen(ime_patch_key) + 1
-            && strncmp(str, ime_patch_key, strlen(ime_patch_key)) == 0
-            && str[len - 1] == '\"'
-        )
-    {
+        && strncmp(str, ime_patch_key, strlen(ime_patch_key)) == 0
+        && str[len - 1] == '\"') {
         sscanf(str,"ime_patch_key = \"%s\"", cloudpinyin->key);
         cloudpinyin->initialized = true;
         cloudpinyin->key[SOGOU_KEY_LENGTH] = '\0';
@@ -851,145 +903,10 @@ void SogouParseKey(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue)
     free(str);
 }
 
-char* SogouParsePinyin(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue)
-{
-    char *start = NULL, *end = NULL;
-    if ((start = strchr(queue->str, '"')) != NULL && (end = strstr(queue->str, "%EF%BC%9A")) != NULL)
-    {
-        start ++;
-        if (start < end)
-        {
-            size_t length = end - start;
-            int conv_length;
-            char *unescapedstring = curl_easy_unescape(queue->curl, start, length, &conv_length);
-            char *realstring = strdup(unescapedstring);
-            curl_free(unescapedstring);
-            return realstring;
-        }
-    }
-    return NULL;
-}
-
-void QQParseKey(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue)
-{
-    char* str = fcitx_utils_trim(queue->str);
-    const char* ime_patch_key = "{\"key\":\"";
-    if (strncmp(str, ime_patch_key, strlen(ime_patch_key)) == 0)
-    {
-        if (sscanf(str,"{\"key\":\"%32s\",\"ret\":\"suc\"}", cloudpinyin->key) > 0)
-        {
-            cloudpinyin->initialized = true;
-            cloudpinyin->key[QQ_KEY_LENGTH] = '\0';
-        }
-    }
-
-    free(str);
-}
-
-char* QQParsePinyin(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue)
-{
-    char *start = NULL, *end = NULL;
-    if ((start = strstr(queue->str, "\"rs\":[\"")) != NULL)
-    {
-        start += strlen( "\"rs\":[\"");
-        if ((end = strstr(start, "\"")) != NULL)
-        {
-            size_t length = end - start;
-            char *realstring = fcitx_utils_malloc0(sizeof(char) * (length + 1));
-            strncpy(realstring, start, length);
-            realstring[length] = '\0';
-            return realstring;
-        }
-    }
-    return NULL;
-}
-
-char* GoogleParsePinyin(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue)
-{
-    char *start = NULL, *end = NULL;
-    if ((start = strstr(queue->str, "\",[\"")) != NULL)
-    {
-        start += strlen( "\",[\"");
-        if ((end = strstr(start, "\"")) != NULL)
-        {
-            size_t length = end - start;
-            char *realstring = fcitx_utils_malloc0(sizeof(char) * (length + 1));
-            strncpy(realstring, start, length);
-            realstring[length] = '\0';
-            return realstring;
-        }
-    }
-    return NULL;
-}
-
-char* BaiduParsePinyin(FcitxCloudPinyin* cloudpinyin, CurlQueue* queue)
-{
-    char *start = NULL, *end = NULL;
-    static iconv_t conv = 0;
-    if (conv == 0)
-        conv = iconv_open("utf-8", "utf-16be");
-
-    if (conv == (iconv_t)(-1))
-        return NULL;
-    if ((start = strstr(queue->str, "[[[\"")) != NULL)
-    {
-        start += strlen( "[[[\"");
-        if ((end = strstr(start, "\",")) != NULL)
-        {
-            size_t length = end - start;
-            if (length % 6 != 0 || length == 0)
-                return NULL;
-
-            size_t i = 0, j = 0;
-            char* buf = fcitx_utils_malloc0((length / 6 + 1) * 2);
-            while (i < length)
-            {
-                if (start[i] == '\\' && start[i+1] == 'u')
-                {
-                    if (ishex(start[i+2]) && ishex(start[i+3]) && ishex(start[i+4]) && ishex(start[i+5]))
-                    {
-                        buf[j++] = (tohex(start[i+2]) << 4) | tohex(start[i+3]);
-                        buf[j++] = (tohex(start[i+4]) << 4) | tohex(start[i+5]);
-                    }
-                    else
-                        break;
-                }
-
-                i += 6;
-            }
-
-            if (i != length)
-            {
-                free(buf);
-                return NULL;
-            }
-            buf[j++] = 0;
-            buf[j++] = 0;
-            size_t len = UTF8_MAX_LENGTH * (length / 6) * sizeof(char);
-            char* realstring = fcitx_utils_malloc0(UTF8_MAX_LENGTH * (length / 6) * sizeof(char));
-            IconvStr p = buf; char *pp = realstring;
-            iconv(conv, &p, &j, &pp, &len);
-
-            free(buf);
-            if (fcitx_utf8_check_string(realstring))
-                return realstring;
-            else
-            {
-                free(realstring);
-                return NULL;
-            }
-        }
-    }
-    return NULL;
-}
-
-
-
 void CloudPinyinHookForNewRequest(void* arg)
 {
     FcitxCloudPinyin* cloudpinyin = (FcitxCloudPinyin*) arg;
-    if (!cloudpinyin->initialized && !cloudpinyin->isrequestkey)
-    {
+    if (!cloudpinyin->initialized && !cloudpinyin->isrequestkey) {
         CloudPinyinRequestKey(cloudpinyin);
     }
 }
